<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Database.inc.php                                               02/20/2003
 * This script contains a set of functions that may be useful when dealing with
 * Virtual Advanced database groups, databases, and messages.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2003 Steve Winn. All Rights Reserved.
 *
 * The source code contains a fair amount of comments, which may aid anyone
 * attempting to follow the program or to modify it.
 *
 * For licensing and copyright information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('va-functions.inc.php');       // A VADV-PHP script.


/**
* Va-Database.inc.php :: FindTopicInfo()
*
* This function will retrieve the variables for the selected database topic using
* the passed file string. This function is identical to the GetTopicInfo function
* except you MUST pass the file string to it. This is useful if you are going to
* search the topics multiple times.
*
* @param string $type
*/
function FindTopicInfo($topicNo, $fileString, $type='') {
    if ((!is_int($topicNo)) && (!is_string($topicNo))) trigger_error(VA_Error(1));
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ($topicNo < 1) $topicNo = 1;

    global $VA_SYSPATHS;                    // Include global variable.

    $type  = strtoupper($type);             // Change the type to uppercase.
    $topic = chr($topicNo + 64);            // Change the topic number into the topic letter.

    $topicInfo = FALSE;
    $offset    = 128;                       // Set the file offset.
    $position  = 2;                         // Set the starting position.
    $dbType    = 'tmp';
    $string    = 'tmp';
    $counter   = 1;

    while ($dbType != '') {
        $dbType = substr($fileString, $position - 2, 1);
        if ($type != '') {
            if ($type == $dbType) {
                $string = rtrim(substr($fileString, $position - 1, 1));
                $string = strtoupper($string);
                if ($string == $topic) {
                    $offset = 128 * ($counter - 1);
                    $topicInfo['topicno'] = $topicNo;
                    $topicInfo['topic']   = $topic;
                    $topicInfo['dbtype']  = GetString($fileString, 1 + $offset, 1);
                    $topicInfo['topic']   = GetString($fileString, 2 + $offset, 1);
                    $topicInfo['desc']    = GetString($fileString, 3 + $offset, 65);
                    $topicInfo['marker1'] = GetLongInt($fileString, 70 + $offset);
                    $topicInfo['marker2'] = GetLongInt($fileString, 74 + $offset);
                    $topicInfo['dbno']    = GetDBCount($topicNo);
                    $dbType = '';
                }
            }
        } else {
            $string = rtrim(substr($fileString, $position - 1, 1));
            $string = strtoupper($string);
            if ($string == $topic) {
                $offset = 128 * ($counter - 1);
                $topicInfo['topicno'] = $topicNo;
                $topicInfo['topic']   = $topic;
                $topicInfo['dbtype']  = GetString($fileString, 1 + $offset, 1);
                $topicInfo['topic']   = GetString($fileString, 2 + $offset, 1);
                $topicInfo['desc']    = GetString($fileString, 3 + $offset, 65);
                $topicInfo['marker1'] = GetLongInt($fileString, 70 + $offset);
                $topicInfo['marker2'] = GetLongInt($fileString, 74 + $offset);
                $topicInfo['dbno']    = GetDBCount($topicNo);
                $dbType = '';
            }
        }
        $position = $position + $offset;
        $counter++;
    }

    return $topicInfo;                      // Return the topic information.
}


/**
* Va-Database.inc.php :: GetTopicInfo()
*
* This function will retrieve the variables for the selected database topic.
*
* @param string $type
*/
function GetTopicInfo($topicNo, $type='') {
    if ((!is_int($topicNo)) && (!is_string($topicNo))) trigger_error(VA_Error(1));
    if ($topicNo < 1) $topicNo = 1;

    global $VA_SYSPATHS;                    // Include global variable.
    
    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '\dbgroup.cfg';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.

    $type  = strtoupper($type);             // Change the type to uppercase.
    $topic = chr($topicNo + 64);            // Change the topic number into the topic letter.

    $topicInfo = FALSE;
    $offset    = 128;                       // Set the file offset.
    $position  = 2;                         // Set the starting position.
    $dbType    = 'tmp';
    $string    = 'tmp';
    $counter   = 1;

    while ($dbType != '') {
        $dbType = substr($fileString, $position - 2, 1);
        if ($type != '') {
            if ($type == $dbType) {
                $string = rtrim(substr($fileString, $position - 1, 1));
                $string = strtoupper($string);
                if ($string == $topic) {
                    $offset = 128 * ($counter - 1);
                    $topicInfo['topicno'] = $topicNo;
                    $topicInfo['topic']   = $topic;
                    $topicInfo['dbtype']  = GetString($fileString, 1 + $offset, 1);
                    $topicInfo['topic']   = GetString($fileString, 2 + $offset, 1);
                    $topicInfo['desc']    = GetString($fileString, 3 + $offset, 65);
                    $topicInfo['marker1'] = GetLongInt($fileString, 70 + $offset);
                    $topicInfo['marker2'] = GetLongInt($fileString, 74 + $offset);
                    $topicInfo['dbno']    = GetDBCount($topicNo);
                    $dbType = '';
                }
            }
        } else {
            $string = rtrim(substr($fileString, $position - 1, 1));
            $string = strtoupper($string);
            if ($string == $topic) {
                $offset = 128 * ($counter - 1);
                $topicInfo['topicno'] = $topicNo;
                $topicInfo['topic']   = $topic;
                $topicInfo['dbtype']  = GetString($fileString, 1 + $offset, 1);
                $topicInfo['topic']   = GetString($fileString, 2 + $offset, 1);
                $topicInfo['desc']    = GetString($fileString, 3 + $offset, 65);
                $topicInfo['marker1'] = GetLongInt($fileString, 70 + $offset);
                $topicInfo['marker2'] = GetLongInt($fileString, 74 + $offset);
                $topicInfo['dbno']    = GetDBCount($topicNo);
                $dbType = '';
            }
        }
        $position = $position + $offset;
        $counter++;
    }

    return $topicInfo;                      // Return the topic information.
}


/**
* Va-Database.inc.php :: GetTopics()
*
* This function will retrieve the topic numbers and store them into an array.
*
* @param string $type
*/
function GetTopics($type='') {
    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '\dbgroup.cfg';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.

    $type = strtoupper($type);              // Change the topic type into uppercase.

    $offset     = 128;                      // Set the file offset.
    $position   = 2;                        // Set the starting position.
    $string     = 'tmp';
    $topics     = array('');
    $counter    = 1;

    while ($string != '') {
        $string = substr($fileString, $position - 2, 1);
        if ($type != '') {
            if ($type == $string) {
                if ($string != '') {
                    $group = rtrim(substr($fileString, $position - 1, 1));
                    $group = strtoupper($group);
                    $group = ord($group) - 64;
                    if (GetDbCount($group) > 0) array_push($topics, $group);
                }
            }
        } else {
                if ($string != '') {
                    $group = rtrim(substr($fileString, $position - 1, 1));
                    $group = strtoupper($group);
                    $group = ord($group) - 64;
                    if (GetDbCount($group) > 0) array_push($topics, $group);
                }
        }
        $counter++;
        $position = $position + $offset;
    }

    return $topics;                        // Return the topic numbers.
}


/**
* Va-Database.inc.php :: GetTopicCount()
*
* This function will determine the total number of topics and return that value.
*
* @param string $type
*/
function GetTopicCount($type = '') {
    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '\dbgroup.cfg';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.

    $type = strtoupper($type);              // Change the topic type into uppercase.

    $offset     = 128;                      // Set the file offset.
    $position   = 2;                        // Set the starting position.
    $string     = 'tmp';
    $topicNo    = 0;
    $counter    = 1;

    while ($string != '') {
        $string = substr($fileString, $position - 2, 1);
        if ($type != '') {
            if ($type == $string) {
                if ($string != '') {
                    $group = rtrim(substr($fileString, $position - 1, 1));
                    $group = strtoupper($group);
                    $group = ord($group) - 64;
                    if (GetDbCount($group) > 0) $topicNo++;
                }
            }
        } else {
                if ($string != '') {
                    $group = rtrim(substr($fileString, $position - 1, 1));
                    $group = strtoupper($group);
                    $group = ord($group) - 64;
                    if (GetDbCount($group) > 0) $topicNo++;
                }
        }
        $counter++;
        $position = $position + $offset;
    }

    return $topicNo;                        // Return the topic count.
}


/**
* Va-Database.inc.php :: FindDbInfo()
*
* This function will retrieve the variables for the selected database from the
* passed fileString. This function is identical to GetDbInfo except you MUST
* pass the fileString. This function is more useful if you are going to be
* searching the same database multiple times.
*
*/
function FindDbInfo($topicNo, $db, $fileString) {
    if ((!is_int($topicNo)) && (!is_string($topicNo))) trigger_error(VA_Error(1));
    if ((!is_int($db))      && (!is_string($db)))      trigger_error(VA_Error(1));
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ($topicNo < 1) $topicNo = 1;
    if ($db < 1) $db = 1;

    $topic = chr($topicNo + 64);            // Convert the topic number into the topic letter.

    global $VA_SYSPATHS;                    // Include the global variable.

    $dbInfo   = array();
    $offset   = 131;
    $position = 75;
    $string   = 'tmp';
    $num      = 0;
    $counter  = 1;

    while ($string != '') {
        $string   = rtrim(substr($fileString, $position - 1, 1));
        $string   = strtoupper($string);    // Convert the topic letter to uppercase.
        $position = $position + 131;
        if ($string == $topic) {
            $num++;
            if ($num == $db) {
                $offset = 131 * ($counter - 1);
                $dbInfo['topicno']    = $topicNo;
                $dbInfo['topic']      = $topic;
                $dbInfo['dbno']       = $db;
                $dbInfo['filename']   = GetString($fileString, 1 + $offset, 8);
                $dbInfo['filepath']   = GetString($fileString, 9 + $offset, 32);
                $dbInfo['desc']       = GetString($fileString, 41 + $offset, 32);
                $dbInfo['private']    = GetInteger($fileString, 73 + $offset);
                $dbInfo['dbgroup']    = GetString($fileString, 75 + $offset, 1);
                $dbInfo['maxmsg']     = GetInteger($fileString, 76 + $offset);
                $dbInfo['minreadsl']  = GetInteger($fileString, 78 + $offset);
                $dbInfo['minwritesl'] = GetInteger($fileString, 80 + $offset);
                $dbInfo['agelimit']   = GetInteger($fileString, 82 + $offset);
                $dbInfo['accflags']   = GetAccFlags(GetLongInt($fileString, 84 + $offset));
                $dbInfo['tagline']    = GetInteger($fileString, 88 + $offset);
                $dbInfo['dispname']   = GetInteger($fileString, 90 + $offset);
                $dbInfo['randtitle']  = GetInteger($fileString, 92 + $offset);
                $dbInfo['qwkconfno']  = GetInteger($fileString, 94 + $offset);
                $dbInfo['moderator']  = GetInteger($fileString, 96 + $offset);
                $dbInfo['validate']   = GetInteger($fileString, 98 + $offset);
                $dbInfo['freqable']   = GetInteger($fileString, 100 + $offset);
                $dbInfo['devicetype'] = GetInteger($fileString, 102 + $offset);
                $dbInfo['dbtype']     = GetString($fileString, 104 + $offset, 1);
                $dbInfo['msgno']      = GetMessageCount($dbInfo);

                $string = '';
            }
        }
        $counter++;
    }

    return $dbInfo;                         // Return the database information.
}


/**
* Va-Database.inc.php :: GetDbInfo()
*
* This function will retrieve the variables for the selected database.
*
*/
function GetDbInfo($topicNo, $db) {
    if ((!is_int($topicNo)) && (!is_string($topicNo))) trigger_error(VA_Error(1));
    if ((!is_int($db))      && (!is_string($db)))      trigger_error(VA_Error(1));
    if ($topicNo < 1) $topicNo = 1;
    if ($db < 1) $db = 1;

    $topic = chr($topicNo + 64);            // Convert the topic number into the topic letter.

    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '\database.cfg';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.

    $dbInfo   = array();
    $offset   = 131;
    $position = 75;
    $string   = 'tmp';
    $num      = 0;
    $counter  = 1;
    
    while ($string != '') {
        $string   = rtrim(substr($fileString, $position - 1, 1));
        $string   = strtoupper($string);    // Convert the topic letter to uppercase.
        $position = $position + 131;
        if ($string == $topic) {
            $num++;
            if ($num == $db) {
                $offset = 131 * ($counter - 1);
                $dbInfo['topicno']    = $topicNo;
                $dbInfo['topic']      = $topic;
                $dbInfo['dbno']       = $db;
                $dbInfo['filename']   = GetString($fileString, 1 + $offset, 8);
                $dbInfo['filepath']   = GetString($fileString, 9 + $offset, 32);
                $dbInfo['desc']       = GetString($fileString, 41 + $offset, 32);
                $dbInfo['private']    = GetInteger($fileString, 73 + $offset);
                $dbInfo['dbgroup']    = GetString($fileString, 75 + $offset, 1);
                $dbInfo['maxmsg']     = GetInteger($fileString, 76 + $offset);
                $dbInfo['minreadsl']  = GetInteger($fileString, 78 + $offset);
                $dbInfo['minwritesl'] = GetInteger($fileString, 80 + $offset);
                $dbInfo['agelimit']   = GetInteger($fileString, 82 + $offset);
                $dbInfo['accflags']   = GetAccFlags(GetLongInt($fileString, 84 + $offset));
                $dbInfo['tagline']    = GetInteger($fileString, 88 + $offset);
                $dbInfo['dispname']   = GetInteger($fileString, 90 + $offset);
                $dbInfo['randtitle']  = GetInteger($fileString, 92 + $offset);
                $dbInfo['qwkconfno']  = GetInteger($fileString, 94 + $offset);
                $dbInfo['moderator']  = GetInteger($fileString, 96 + $offset);
                $dbInfo['validate']   = GetInteger($fileString, 98 + $offset);
                $dbInfo['freqable']   = GetInteger($fileString, 100 + $offset);
                $dbInfo['devicetype'] = GetInteger($fileString, 102 + $offset);
                $dbInfo['dbtype']     = GetString($fileString, 104 + $offset, 1);
                $dbInfo['msgno']      = GetMessageCount($dbInfo);
                
                $string = '';
            }
        }
        $counter++;
    }

    return $dbInfo;                         // Return the database information.
}


/**
* Va-Database.inc.php :: GetDbCount()
*
* This function will get the number of available databases in the selected
* topic. It will filter out those which the user does not have access to.
*
*/
function GetDbCount($topic) {
    if ((!is_int($topic)) && (!is_string($topic))) trigger_error(VA_Error(1));
    if ($topic < 0) trigger_error(VA_Error(2));

    $topic = chr($topic + 64);

    global $VA_SYSPATHS;                    // Include the global variable.

    $file = $VA_SYSPATHS['main'] . '\database.cfg';
    if (!is_file($file)) return FALSE;
    
    $fileString = ReadCfgFile($file);       // Read the file.

    $offset     = 131;                      // Set the file offset.
    $position   = 75;                       // Set the starting position.
    $string     = 'tmp';
    $dbNo       = 0;
    $counter    = 1;

    while ($string != '') {
        $string = rtrim(substr($fileString, $position - 1, 1));
        $string = strtoupper($string);
        $position = $position + 131;
        if ($string == $topic) {
            $dbInfo = array();
            $offset = 131 * ($counter - 1);
            $dbInfo['minreadsl'] = GetInteger($fileString, 78 + $offset);
            $dbInfo['agelimit']  = GetInteger($fileString, 82 + $offset);
            $dbInfo['accflags']  = GetLongInt($fileString, 84 + $offset);
            if (DbAccess($dbInfo) != 0) {
                $dbNo++;
            }
        }
        $counter++;
    }

    return $dbNo;                           // Return the database count.
}


/**
* Va-Database.inc.php :: FindMessageInfo()
*
* This function will retrieve the variables for the specified message using the
* passed fileString. This function is identical to the GetMessageInfo function
* except you must pass the fileString. This is useful if you are going to be
* searching the same message database multiple times.
*
*/
function FindMessageInfo($dbInfo, $msgNo, $fileString) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    if ((!is_int($msgNo))   && (!is_string($msgNo))) trigger_error(VA_Error(1));
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ($msgNo < 1) $msgno = 1;
    if ($msgNo > $dbInfo['msgno']) return FALSE;

    global $VA_SYSPATHS;                    // Include global variable.

    $msgInfo = array();
    $offset  = 582 * ($msgNo - 1);
    $msgInfo['topicno']     = $dbInfo['topicno'];
    $msgInfo['topic']       = $dbInfo['topic'];
    $msgInfo['dbno']        = $dbInfo['dbno'];
    $msgInfo['fromuserno']  = GetLongInt($fileString, 1 + $offset);
    $msgInfo['touserno']    = GetLongInt($fileString, 5 + $offset);
    $msgInfo['fromhandle']  = GetString($fileString, 9 + $offset, 40);
    if ($msgInfo['fromhandle'] == '') $msgInfo['fromhandle'] = 'Unknown';
    $msgInfo['tohandle']    = GetString($fileString, 49 + $offset, 40);
    if ($msgInfo['tohandle'] == '') $msgInfo['tohandle'] = 'All';
    $msgInfo['fromnetnode'] = GetString($fileString, 89 + $offset, 80);
    $msgInfo['tonetnode']   = GetString($fileString, 169 + $offset, 80);
    $msgInfo['fromnetid']   = GetInteger($fileString, 249 + $offset);
    $msgInfo['tonetid']     = GetInteger($fileString, 251 + $offset);
    $msgInfo['daysold']     = GetInteger($fileString, 257 + $offset);
    $msgInfo['subject']     = GetString($fileString, 269 + $offset, 64);
    $msgInfo['creation']    = GetString($fileString, 335 + $offset, 38);
    $msgInfo['date']        = GetString($fileString, 335 + $offset, 16);
    $msgInfo['time']        = GetString($fileString, 352 + $offset, 8);
    $msgInfo['timezone']    = GetString($fileString, 361 + $offset, 12);
    $msgInfo['offline']     = GetInteger($fileString, 375 + $offset);
    $msgInfo['attfilesize'] = GetLongInt($fileString, 379 + $offset);
    $msgInfo['attfilename'] = GetString($fileString, 383 + $offset, 12);
    $msgInfo['downloadno']  = GetInteger($fileString, 463 + $offset);
    $msgInfo['filename']    = GetString($fileString, 471 + $offset, 8);
    $msgInfo['msgpos']      = GetLongInt($fileString, 479 + $offset);
    $msgInfo['msglength']   = GetLongInt($fileString, 483 + $offset);

    return $msgInfo;                        // Return the message information.
}


/**
* Va-Database.inc.php :: GetMessageInfo()
*
* This function will retrieve the variables for the specified message.
*
*/
function GetMessageInfo($dbInfo, $msgNo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    if ((!is_int($msgNo))   && (!is_string($msgNo))) trigger_error(VA_Error(1));
    if ($msgNo < 1) $msgno = 1;
    if ($msgNo > $dbInfo['msgno']) return FALSE;

    global $VA_SYSPATHS;                    // Include global variable.

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . "\\" . $dbInfo['filename'];

    $datFile = $file . '.DAT';              // Set the database filename.
    if (!is_file($datFile)) return FALSE;

    $fileString = ReadCfgFile($datFile);    // Read the file.

    $msgInfo = array();
    $offset  = 582 * ($msgNo - 1);
    $msgInfo['topicno']     = $dbInfo['topicno'];
    $msgInfo['topic']       = $dbInfo['topic'];
    $msgInfo['dbno']        = $dbInfo['dbno'];
    $msgInfo['fromuserno']  = GetLongInt($fileString, 1 + $offset);
    $msgInfo['touserno']    = GetLongInt($fileString, 5 + $offset);
    $msgInfo['fromhandle']  = GetString($fileString, 9 + $offset, 40);
    if ($msgInfo['fromhandle'] == '') $msgInfo['fromhandle'] = 'Unknown';
    $msgInfo['tohandle']    = GetString($fileString, 49 + $offset, 40);
    if ($msgInfo['tohandle'] == '') $msgInfo['tohandle'] = 'All';
    $msgInfo['fromnetnode'] = GetString($fileString, 89 + $offset, 80);
    $msgInfo['tonetnode']   = GetString($fileString, 169 + $offset, 80);
    $msgInfo['fromnetid']   = GetInteger($fileString, 249 + $offset);
    $msgInfo['tonetid']     = GetInteger($fileString, 251 + $offset);
    $msgInfo['daysold']     = GetInteger($fileString, 257 + $offset);
    $msgInfo['subject']     = GetString($fileString, 269 + $offset, 64);
    $msgInfo['creation']    = GetString($fileString, 335 + $offset, 38);
    $msgInfo['date']        = GetString($fileString, 335 + $offset, 16);
    $msgInfo['time']        = GetString($fileString, 352 + $offset, 8);
    $msgInfo['timezone']    = GetString($fileString, 361 + $offset, 12);
    $msgInfo['offline']     = GetInteger($fileString, 375 + $offset);
    $msgInfo['attfilesize'] = GetLongInt($fileString, 379 + $offset);
    $msgInfo['attfilename'] = GetString($fileString, 383 + $offset, 12);
    $msgInfo['downloadno']  = GetInteger($fileString, 463 + $offset);
    $msgInfo['filename']    = GetString($fileString, 471 + $offset, 8);
    $msgInfo['msgpos']      = GetLongInt($fileString, 479 + $offset);
    $msgInfo['msglength']   = GetLongInt($fileString, 483 + $offset);

    return $msgInfo;                        // Return the message information.
}


/**
* Va-Database.inc.php :: GetMessage()
*
* This function will retrieve the message for the selected information.
*
*/
function GetMessage($dbInfo, $msgNo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    if ((!is_int($msgNo))   && (!is_string($msgNo))) trigger_error(VA_Error(1));
    if ($msgNo < 1) $msgNo = 1;
    if ($msgNo > $dbInfo['msgno']) return FALSE;

    global $VA_SYSPATHS;                    // Include the global variable.
    
    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . "\\" . $dbInfo['filename'];

    $datFile = $file . '.DAT';
    if (!is_file($datFile)) return FALSE;

    $binFile = $file . '.BIN';
    if (!is_file($binFile)) return FALSE;

    $fileString = ReadCfgFile($datFile);    // Read the file.

    $offset    = 582 * ($msgNo - 1);                        // Set the file offset.
    $msgPos    = GetLongInt($fileString, 479 + $offset) - 1;// Determine the message position.
    $msgLength = GetLongInt($fileString, 483 + $offset);    // Determine the message length.

    $fp = fopen($binFile, 'r');                             // Open the file.
    fseek($fp, $msgPos);
    $message = StripRouting(fread($fp, $msgLength));        // Get the message.
    fclose($fp);                                            // Close the file.
    
    $message = trim(strip_tags($message));                  // Strip any tags.

    return $message;                        // Return the message.
}


/**
* Va-Database.inc.php :: GetMessageCount()
*
* This function will determine the number of messages contained within the
* specified database.
*
*/
function GetMessageCount($dbInfo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));

    global $VA_SYSPATHS;                    // Include the global variable.
    
    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . "\\" . $dbInfo['filename'];

    $datFile = $file . '.DAT';              // Add the datafile extension.
    if (!is_file($datFile)) return FALSE;

    $fileSize = filesize($datFile);         // Determine the size of the file.
    $msgNo    = intval($fileSize / 582);    // Determine the number of messages.

    return $msgNo;                          // Return the message count.
}


/**
* Va-Database.inc.php :: GetMessagesTo()
*
* This function will scan the current database for all messages to the current
* user and store them into an array.
*
*/
function SearchMessagesTo($dbInfo, $userNo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    if ((!is_int($userNo))  && (!is_string($userNo))) trigger_error(VA_Error(1));
    if ($userNo == 0) {
        $msgTo[1] = '';
        return $msgTo;
    }

    include_once('va-networks.inc.php');    // Include network information.

    // This function has been optimized for speed and does not use the pre-
    // written functions above.
    global $VA_SYSPATHS;                    // Include global variable.

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . "\\" . $dbInfo['filename'];

    $datFile = $file . '.DAT';              // Set the database filename.
    if (!is_file($datFile)) return FALSE;

    $netInfo    = GetNetworkInfo();
    $fileString = ReadCfgFile($datFile);    // Read the database file.

    $msgNo   = 1;                             // Starting point.
    $cnt     = 1;                             // Counter.
    $msgTo   = array();

    while ($msgNo <= $dbInfo['msgno']) {
        $offset = 582 * ($msgNo - 1);
        $msgInfo['touserno']  = GetLongInt($fileString, 5 + $offset);
        if ($msgInfo['touserno'] == $userNo) {
            $msgInfo['tonetid'] = GetInteger($fileString, 251 + $offset);
            if (isset($netInfo[$msgInfo['tonetid']]['netno'])) {
                if ($msgInfo['tonetid'] == $netInfo[$msgInfo['tonetid']]['netno']) {
                    $msgInfo['tonetnode'] = GetString($fileString, 169 + $offset, 80);
                    if ($msgInfo['tonetnode'] == $netInfo[$msgInfo['tonetid']]['settings']['systemid']) {
                        $msgTo[$cnt] = $msgNo;      // Add the message number to the array.
                        $cnt++;                     // Increase the counter by one.
                    }
                }
            }
        }
        $msgNo++;
    }

    $msgTo[0] = $cnt - 1;                 // Set the total number of messages.

    return $msgTo;                        // Return the messages from this user.
}


/**
* Va-Database.inc.php :: GetMessagesFrom()
*
* This function will scan the current database for all messages from the
* current user and store them into an array.
*
*/
function SearchMessagesFrom($dbInfo, $userNo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    if ((!is_int($userNo))  && (!is_string($userNo))) trigger_error(VA_Error(1));
    if ($userNo == 0) {
        $msgFrom[1] = '';
        return $msgFrom;
    }
    
    include_once('va-networks.inc.php');    // Include network information.
    
    // This function has been optimized for speed and does not use the pre-
    // written functions above.
    global $VA_SYSPATHS;                    // Include global variable.

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . "\\" . $dbInfo['filename'];
    $datFile = $file . '.DAT';              // Set the database filename.

    if (!is_file($datFile)) return FALSE;

    $netInfo    = GetNetworkInfo();
    $fileString = ReadCfgFile($datFile);    // Read the database file.

    $msgNo   = 1;                             // Starting point.
    $cnt     = 1;                             // Counter.
    $msgFrom = array();
    
    while ($msgNo <= $dbInfo['msgno']) {
        $offset = 582 * ($msgNo - 1);
        $msgInfo['fromuserno']  = GetLongInt($fileString, 1 + $offset);
        if ($msgInfo['fromuserno'] == $userNo) {
            $msgInfo['fromnetid'] = GetInteger($fileString, 249 + $offset);
            if (isset($netInfo[$msgInfo['fromnetid']]['netno'])) {
                if ($msgInfo['fromnetid'] == $netInfo[$msgInfo['fromnetid']]['netno']) {
                    $msgInfo['fromnetnode'] = GetString($fileString, 89 + $offset, 80);
                    if ($msgInfo['fromnetnode'] == $netInfo[$msgInfo['fromnetid']]['settings']['systemid']) {
                        $msgFrom[$cnt] = $msgNo;    // Add the message number to the array.
                        $cnt++;                     // Increase the counter by one.
                    }
                }
            }
        }
        $msgNo++;
    }
    
    $msgFrom[0] = $cnt - 1;                 // Set the total number of messages.
    
    return $msgFrom;                        // Return the messages from this user.
}


/**
* Va-Database.inc.php :: GetEmailInfo()
*
* This function will return the email database information in the same format
* as dbInfo.
*
*/
function GetEmailInfo() {
    $emailInfo['filename']   = 'email';
    $emailInfo['filepath']   = '';
    $emailInfo['desc']       = 'Private Email Database';
    $emailInfo['private']    = 0;
    $emailInfo['dbgroup']    = '';
    $emailInfo['maxmsg']     = 0;
    $emailInfo['minreadsl']  = 0;
    $emailInfo['minwritesl'] = 0;
    $emailInfo['agelimit']   = 0;
    $emailInfo['accflags']   = 0;
    $emailInfo['tagline']    = 0;
    $emailInfo['dispname']   = 0;
    $emailInfo['randtitle']  = 0;
    $emailInfo['qwkconfno']  = 0;
    $emailInfo['moderator']  = 0;
    $emailInfo['validate']   = 0;
    $emailInfo['freqable']   = 0;
    $emailInfo['devicetype'] = 0;
    $emailInfo['dbtype']     = 'E';
    $emailInfo['msgno']      = GetMessageCount($emailInfo);

    return $emailInfo;                      // Return the email database information.
}


/**
* Va-Database.inc.php :: GetLastPostInfo()
*
* This function will get the last post's message information of the
* specified database.
*
*/
function GetLastPostInfo($dbInfo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    
    $lastPost = GetMessageInfo($dbInfo, $dbInfo['msgno']);

    return $lastPost;                       // Return the lastpost information.
}


/**
* Va-Database.inc.php :: DbAccess()
*
* This function will determine if the user has access clearance for the specified
* database.
*
*/
function DbAccess($dbInfo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    
    global $USER;                           // Include the global variable.
    $access = FALSE;                        // Set the default access value, false.
    
    if ($USER['securitylevel'] >= $dbInfo['minreadsl']) {
        if ($USER['age'] >= $dbInfo['agelimit']) {
            if (($dbInfo['accflags'] == '') ||
                (strcmp(trim($USER['accflags']), trim($dbInfo['accflags'])) == 0) ||
                (strpos(trim($USER['accflags']), 'A')) ||
                (trim($USER['accflags']) == trim($dbInfo['accflags']))) {
                $access = TRUE;
            }
        }
    }
    
    return $access;                         // Return the access result.
}

?>
