<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Download.inc.php                                               02/20/2003
 * This script will retrieve all of the informaton related to the configured
 * download protocols that are found in DOWNLOAD.CFG.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2003 Steve Winn. All Rights Reserved.
 *
 * The source code contains a fair amount of comments, which may aid anyone
 * attempting to follow the program or to modify it.
 *
 * For licensing and copyright information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('va-functions.inc.php');       // A VADV-PHP script.


/**
* Va-Download.inc.php :: GetDownload()
*
* Retrieve all of the download protocol information and store it into a
* multidimensional array.
*
* Array format: $download[1]['setting']
*               $download[0] = Total number of download protocols.
*
*/
function GetDownload() {
    $fileString = ReadDownload();
    if (!$fileString) trigger_error(VA_Error(3));

    $string   = 'tmp';
    $download = array();                    // Create the array.

    for ($x = 0; $string <> ''; $x++) {
        $offset = 73 * $x;
        $string = GetString($fileString, $offset + 1, 1);
        if ($string <> '') {
            $download[$x + 1]['key']     = GetString($fileString, 1 + $offset, 1);
            $download[$x + 1]['desc']    = GetString($fileString, 2 + $offset, 23);
            $download[$x + 1]['batch']   = GetInteger($fileString, 25 + $offset);
            $download[$x + 1]['command'] = GetString($fileString, 26 + $offset, 48);
        }
    }

    $download[0] = $x - 1;                  // Put the number of doors in position 0.

    return $download;                       // Return the upload protocols.

}


/**
* Va-Download.inc.php :: PutDownload()
*
* This function will change the download.cfg to match the configuration variable
* passed. It then returns the new file string. The configuration variable MUST
* be in the format of $variable[1]['setting'].
*
*/
function PutDownload($fileString, $configVariable) {
    if (!is_string($fileString))    trigger_error(VA_Error(1));
    if (!is_array($configVariable)) trigger_error(VA_Error(1));

    $length  = count($configVariable);      // Get the number of elements in the config variable.
    $pointer = 0;                           // Reset the pointer.

    for ($x = 0; $pointer < $length; $x++) {
        $offset = 73 * $x;
        if (isset($configVariable[$x + 1])) {
            $pointer++;
            if (isset($configVariable[$x + 1]['key']))     $fileString = PutString($fileString, $configVariable[$x + 1]['key'], 1 + $offset, 1);
            if (isset($configVariable[$x + 1]['desc']))    $fileString = PutString($fileString, $configVariable[$x + 1]['desc'], 2 + $offset, 23);
            if (isset($configVariable[$x + 1]['batch']))   $fileString = PutInteger($fileString, $configVariable[$x + 1]['batch'], 25 + $offset);
            if (isset($configVariable[$x + 1]['command'])) $fileString = PutString($fileString, $configVariable[$x + 1]['command'], 26 + $offset, 48);
        }
    }

    return $fileString;                     // Return the new file string.
}


/**
* Va-Download.inc.php :: ReadDownload()
*
* This function will read the download.cfg file.
*
*/
function ReadDownload() {
    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '\download.cfg';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.

    return $fileString;                     // Return the file string.
}


/**
* Va-Download.inc.php :: WriteDownload()
*
* This function will write a new download.cfg file based on the file string
* passed to it. If the file string is not in the correct format, then your
* download.cfg file will become corrupt and the BBS could stop functioning!
*
*/
function WriteDownload($fileString) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));

    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '\download.cfg';
    if (!is_file($file)) return FALSE;

    $status = WriteCfgFile($file, $fileString);

    return $status;                         // Return the result status.
}


// Retrieve the download protocols automatically.
$VA_DOWNLOAD = GetDownload();

?>
